package com.sec.xinhua.gateway.handler;

import com.alibaba.fastjson.JSON;
import com.sec.xinhua.gateway.beans.*;
import com.sec.xinhua.gateway.utils.ApiHandlerException;
import com.sec.xinhua.gateway.utils.ApiParamTypeCache;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * dubbo參數映射
 *
 * @author zhangqx
 * @version $Id: DubboArgumentConvertHandler.java, v 0.1 2015年12月9日 下午3:43:21 zhangqx Exp $
 */
public class DubboArgumentConvertHandler implements ApiHandler {
    private static final Log logger = LogFactory.getLog("errorLogger");

    @Override
    public ApiResult handle(ApiContext apiContext) {
        if (apiContext.getApi().getParams() == null) {
            apiContext.setInvokeTypes(null);
            apiContext.setInvokeValues(null);
            return null;
        }
        List<SimpleApiParam> apiParams = apiContext.getApi().getParams();
        String[] invokeTypes = new String[apiParams.size()];
        Object[] invokeValues = new Object[apiParams.size()];

        for (int i = 0; i < apiParams.size(); i++) {
            try {
                InvokeParam invokeParam = convert(apiParams.get(i), apiContext);
                invokeTypes[i] = invokeParam.getType();
                invokeValues[i] = invokeParam.getValue();
            } catch (ApiHandlerException e) {
                logger.info("Parse api param error", e);
                return ApiResultCache.getApiResult(ApiCode.API_INVALID_REQUEST, e.getMessage());
            }
        }
        apiContext.setInvokeTypes(invokeTypes);
        apiContext.setInvokeValues(invokeValues);
        return null;
    }

    private InvokeParam convert(SimpleApiParam param, ApiContext apiContext) throws ApiHandlerException {
        if (param.getParamType() == ApiParamType.BYTE_ARRAY.getCode()) {
            return convertByteArray(apiContext, param);
        } else if (param.getParamType() == ApiParamType.MAP.getCode()) {
            return convertMapParam(apiContext, param);
        } else if (param.getParamType() == ApiParamType.List.getCode()) {
            return convertListParam(apiContext, param);
        } else {
            Object httpValue = apiContext.getHttpParam().getParameter(param.getSourceName());
            return convertCommon(param, httpValue);
        }
    }

    /**
     * 解析byte[] 类型
     *
     * @param apiContext
     * @param param
     * @return
     */
    private InvokeParam convertByteArray(ApiContext apiContext, SimpleApiParam param) throws ApiHandlerException {
        SimpleApi api = apiContext.getApi();
        if (StringUtils.isEmpty(param.getClassName())) {
            logger
                    .error("[Invalid api] " + api.getApiName() + "_" + api.getApiVersion() + " byte array class null");
            throw new ApiHandlerException("[Invalid api param]parameter '" + param.getSourceName()
                    + "' is byte[] ,but the class name was null");
        }
        String invokeType = param.getClassName();
        byte[] bytes = (byte[]) apiContext.getHttpParam().getParameter(param.getSourceName());
        return new InvokeParam(invokeType, bytes);
    }

    /**
     * 解析Map类型（风控用）
     *
     * @param apiContext
     * @param param
     * @return
     * @throws ApiHandlerException
     */
    private InvokeParam convertMapParam(ApiContext apiContext, SimpleApiParam param) throws ApiHandlerException {
        SimpleApi api = apiContext.getApi();
        if (StringUtils.isEmpty(param.getClassName())) {
            logger
                    .error("[Invalid api] " + api.getApiName() + "_" + api.getApiVersion() + " mapping object class null");
            throw new ApiHandlerException("[Invalid api param]parameter '" + param.getSourceName()
                    + "' is map ,but the class was null");
        }
        String invokeType = param.getClassName();
        Object mapString = apiContext.getHttpParam().getParameter(param.getSourceName());
        Map<String, Object> properties = JSON.parseObject(String.valueOf(mapString));
        return new InvokeParam(invokeType, properties);
    }

    /**
     * 解析List类型（风控用）
     *
     * @param apiContext
     * @param param
     * @return
     * @throws ApiHandlerException
     */
    private InvokeParam convertListParam(ApiContext apiContext, SimpleApiParam param) throws ApiHandlerException {
        SimpleApi api = apiContext.getApi();
        if (StringUtils.isEmpty(param.getClassName())) {
            logger
                    .error("[Invalid api] " + api.getApiName() + "_" + api.getApiVersion() + " mapping object class null");
            throw new ApiHandlerException("[Invalid api param]parameter '" + param.getSourceName()
                    + "' is list ,but the class was null");
        }
        String invokeType = param.getClassName();
        Object mapString = apiContext.getHttpParam().getParameter(param.getSourceName());
        List<String> properties = JSON.parseArray(String.valueOf(mapString), String.class);
        return new InvokeParam(invokeType, properties);
    }


    /**
     * 解析对象类型
     *
     * @param param
     * @param apiContext
     * @return
     * @throws ApiHandlerException
     */
    private InvokeParam convertObject(ApiContext apiContext, SimpleApiParam param) throws ApiHandlerException {
        SimpleApi api = apiContext.getApi();
        if (StringUtils.isEmpty(param.getClassName())) {
            logger
                    .error("[Invalid api] " + api.getApiName() + "_" + api.getApiVersion() + " mapping object class null");
            throw new ApiHandlerException("[Invalid api param]parameter '" + param.getSourceName()
                    + "' is object ,but the class was null");
        }
        Map<String, Object> properties = new HashMap<String, Object>();
        for (SimpleApiParam f : param.getFields()) {
            Object pValue = apiContext.getHttpParam().getParameter(f.getSourceName());
            InvokeParam invokeParam = convertCommon(f, pValue);
            properties.put(f.getDestName(), invokeParam.getValue());
        }
        return new InvokeParam(param.getClassName(), properties);
    }

    /**
     * 解析普通类型
     *
     * @param param
     * @return
     * @throws ApiHandlerException
     */
    private InvokeParam convertCommon(SimpleApiParam param, Object httpValue) throws ApiHandlerException {

        // 处理默认值
        if (httpValue == null) {
            httpValue = param.getDefaultValue();
            // 默认值兼容
            if (StringUtils.isEmpty((String) httpValue)) {
                if (param.getParamType() >= 10 && param.getParamType() <= 16) {
                    httpValue = 0;// 基本类型
                } else if (param.getParamType() == 17) {
                    httpValue = false;// 布尔类型
                } else if (param.getParamType() >= 20 && param.getParamType() <= 27) {
                    httpValue = null;// 包装类型
                }
            }
        }

        // 得到入参类型
        String invokeType = ApiParamTypeCache.getClassName(param.getParamType());
        if (invokeType == null) {
            logger.error("[Invalid api] Unsupported parameter type:'" + param.getParamType() + ", param name:"
                    + param.getSourceName());
            throw new ApiHandlerException("Unsupported parameter type:'" + param.getParamType() + ", param name:"
                    + param.getSourceName());
        }

        // 得到入参实际值
        Object invokeValue = ApiParamValueConvertor.convert(httpValue, param.getSourceName(), param.getParamType());
        return new InvokeParam(invokeType, invokeValue);
    }
}
